==> fleetback-media-carousel-1.0.4.tgz: package/src/VideoController.js <==
// @flow
import React, { Component } from 'react';
import { observer } from 'mobx-react';
import { PanResponder, PanResponderInstance, Platform, Text, TouchableHighlight, View } from 'react-native';
import {action, makeObservable, observable} from 'mobx';
import EntypoIcon from 'react-native-vector-icons/Entypo';
import AntDesign from 'react-native-vector-icons/AntDesign';
import MaterialIcons from 'react-native-vector-icons/MaterialIcons';
import { contextualStyleSheet } from 'react-native-contextual-dark-mode';
import { SafeAreaView } from 'react-native-safe-area-context';

import { calculateSeekerPosition, formattedDisplayTime } from './VideoPlayerHelper';

export type VideoInformation = {
    duration: number,
    currentTime: number,
    paused: boolean,
    seekerFillWidth: number,
    seekerPosition: number,
    seekerOffset: number,
    seeking: boolean,
    endReached: boolean,
    target: ?string,
};

type VideoPlayerProps = {
    actionRightCallback?: Function,
    actionRightIcon?: Component,
    actionLeftCallback?: Function,
    actionLeftIcon?: Component,
    isDarkMode?: boolean,
    updateVideoInformation: (id: string, info: VideoInformation) => void,
    videoInformation: VideoInformation,
    onSeek: Function,
    id: string,
    seekerWidth: number,
    setSeekerWidth: Function,
    onEnd: Function,
};

@observer
class VideoController extends Component<VideoPlayerProps> {
    @observable dragging = false;
    seekPanResponder: PanResponderInstance;

    constructor(props) {
        super(props);
        makeObservable(this);
        this.initSeekPanResponder();
    }

    @action
    setIsDragging = () => { this.dragging = true; };

    @action
    setNotDragging = () => { this.dragging = false; };

    initSeekPanResponder = () => {
        this.seekPanResponder = PanResponder.create({
            onStartShouldSetPanResponder: () => true,
            onStartShouldSetPanResponderCapture: () => true,
            onMoveShouldSetPanResponder: () => true,
            onMoveShouldSetPanResponderCapture: () => true,
            onPanResponderTerminationRequest: () => false,
            onPanResponderGrant: (evt) => {
                const { updateVideoInformation, videoInformation, id } = this.props;
                this.setSeekerPosition(evt.nativeEvent.locationX, id);
                updateVideoInformation(id, { seeking: true });
                if (videoInformation.endReached) {
                    updateVideoInformation(id, { endReached: false });
                }
            },
            onPanResponderMove: (evt, gestureState) => {
                const {
                    updateVideoInformation,
                    videoInformation: { seeking, paused, seekerOffset, seekerPosition, duration },
                    id,
                    seekerWidth,
                } = this.props;

                if (!seeking) {
                    updateVideoInformation(id, { seeking: true });
                }
                if (!paused && !this.dragging) {
                    updateVideoInformation(id, { paused: true });
                    this.setIsDragging();
                } else {
                    this.setSeekerPosition(seekerOffset + gestureState.dx, id);
                    const timeToSeek = (duration * (seekerPosition / seekerWidth));
                    this.seekTo(timeToSeek);
                    this.clearControlTimeout();
                }
            },
            onPanResponderRelease: () => {
                const {
                    updateVideoInformation,
                    videoInformation: { paused, seekerPosition, duration },
                    id,
                    seekerWidth,
                } = this.props;

                if (this.dragging && paused) {
                    this.setNotDragging();
                    updateVideoInformation(id, { paused: false });
                }

                const timeToSeek = (duration * (seekerPosition / seekerWidth));

                if (timeToSeek >= duration) {
                    this.props.onEnd();
                } else {
                    this.seekTo(timeToSeek);
                    this.clearControlTimeout();
                    this.seekTimeout = setTimeout(() => {
                        updateVideoInformation(id, { seeking: false });
                    }, 150);
                }
            },
        });
    };

    @action
    togglePlayPause = () => {
        const { updateVideoInformation, videoInformation, id, onSeek } = this.props;
        updateVideoInformation(id, { paused: !videoInformation.paused });
        if (videoInformation.endReached) {
            updateVideoInformation(id, { endReached: false });
            if (Platform.OS === 'ios') {
                onSeek(0);
            }
        }
    };

    @action
    setSeekerPosition = (position: number = 0, id: number) => {
        const { updateVideoInformation, videoInformation, seekerWidth } = this.props;
        const pos = Math.max(0, Math.min(seekerWidth, position));
        updateVideoInformation(id, { seekerFillWidth: pos, seekerPosition: pos });
        if (!videoInformation.seeking) {
            updateVideoInformation(id, { seekerOffset: position });
        }
    };

    @action
    seekTo = (time: number = 0) => {
        const { updateVideoInformation, onSeek, id } = this.props;
        updateVideoInformation(id, { currentTime: time });
        onSeek(time);
    };

    onLayout = ({ nativeEvent: { layout: { width } } }) => {
        const { seekerWidth, setSeekerWidth, videoInformation, id } = this.props;
        if (seekerWidth !== width) {
            setSeekerWidth(width);
            if (videoInformation) {
                const position = calculateSeekerPosition(videoInformation.currentTime, videoInformation.duration, seekerWidth);
                this.setSeekerPosition(position, id);
            }
        }
    };

    skipClosure = (skipValue: number) => action(() => {
        const {
            id,
            videoInformation: { currentTime, duration },
            seekerWidth,
            updateVideoInformation,
        } = this.props;
        let endReached = false;

        updateVideoInformation(id, { seeking: true });
        let skippedBack = currentTime + skipValue;
        if (skippedBack < 0) {
            skippedBack = 0;
        } else if (skippedBack > duration) {
            skippedBack = duration;
            endReached = true;
        }
        updateVideoInformation(id, { currentTime: skippedBack, endReached });
        const position = calculateSeekerPosition(skippedBack, duration, seekerWidth);
        this.setSeekerPosition(position, id);
        const timeToSeek = (duration * (position / seekerWidth));
        this.seekTo(timeToSeek);
        this.seekTimeout = setTimeout(() => {
            updateVideoInformation(id, { seeking: false });
        }, 150);
    });

    clearControlTimeout = () => {
        clearTimeout(this.seekTimeout);
    };

    renderSeekbar(color: string, seekerPosition: number) {
        return (
            <View
                style={s.seekbarContainer}
                collapsable={false}
                {...this.seekPanResponder.panHandlers}
            >
                <View
                    style={s.seekbarTrack}
                    onLayout={this.onLayout}
                    pointerEvents="none"
                >
                    <View
                        style={[
                            s.seekbarFill,
                            { width: seekerPosition },
                        ]}
                        pointerEvents="none"
                    />
                </View>
                <View
                    style={[s.seekbarHandle, { left: seekerPosition - 1 }]}
                    pointerEvents="none"
                >
                    <View
                        style={s.seekbarCircle}
                        pointerEvents="none"
                    />
                </View>
            </View>
        );
    }

    renderControl = (callback, children, disabled) => (callback && children ? (
        <TouchableHighlight
            underlayColor="transparent"
            activeOpacity={0.3}
            disabled={disabled}
            onPress={callback}
            style={s.controlButton}
        >
            {children}
        </TouchableHighlight>
    ) : (<View style={s.emptyBox} />));

    renderPlayPause = (paused: boolean, color: string) => this.renderControl(
        this.togglePlayPause,
        paused
            ? <EntypoIcon size={33} name="controller-play" color={color} />
            : <AntDesign size={35} name="pause" color={color} />,
        false,
    );

    renderBackTime = (color: string, disabled: boolean) => this.renderControl(
        this.skipClosure(-5),
        <MaterialIcons size={35} name="replay-5" color={color} />,
        disabled,
    );

    renderForwardTime = (color: string, disabled: boolean) => this.renderControl(
        this.skipClosure(5),
        <MaterialIcons size={35} name="forward-5" color={color} />,
        disabled,
    );

    render() {
        const { videoInformation, actionLeftCallback, actionLeftIcon, actionRightCallback, actionRightIcon } = this.props;
        const { currentTime, duration, seekerPosition, paused }: VideoInformation = videoInformation;
        const color = this.props.isDarkMode ? 'white' : 'black';
        const isVideoLongEnough = duration > 6;
        return (
            <SafeAreaView style={s.footerContainer} edges={['bottom', 'right', 'left']} mode="padding">
                <View style={s.column}>
                    {this.renderSeekbar(color, seekerPosition)}
                    <View style={s.timerContainer}>
                        <Text style={s.timeText}>{formattedDisplayTime(currentTime)}</Text>
                        <Text style={s.timeText}>{formattedDisplayTime(duration)}</Text>
                    </View>
                    <View style={s.bottomControlGroup}>
                        {this.renderControl(actionLeftCallback, actionLeftIcon)}
                        {this.renderBackTime(isVideoLongEnough ? color : SEEKBAR_COLOR, !isVideoLongEnough)}
                        {this.renderPlayPause(paused, color)}
                        {this.renderForwardTime(isVideoLongEnough ? color : SEEKBAR_COLOR, !isVideoLongEnough)}
                        {this.renderControl(actionRightCallback, actionRightIcon)}
                    </View>
                </View>
            </SafeAreaView>
        );
    }
}

const SEEKBAR_COLOR = '#7D7D81';

const s = contextualStyleSheet((darkMode) => ({
    footerContainer: {
        position: 'absolute',
        bottom: 0,
        width: '100%',
        flexDirection: 'row',
        justifyContent: 'center',
        backgroundColor: darkMode ? '#1D252DB3' : '#EDEDEDB3',
    },
    controlButton: {
        paddingVertical: 16,
    },
    emptyBox: {
        width: 35,
        height: 35,
    },
    column: {
        width: '100%',
        flexDirection: 'column',
        alignItems: 'center',
        justifyContent: 'space-between',
    },
    timerContainer: {
        flexDirection: 'row',
        width: '100%',
        justifyContent: 'space-between',
        paddingHorizontal: 20,
    },
    bottomControlGroup: {
        flexDirection: 'row',
        height: null,
        width: null,
        alignSelf: 'stretch',
        alignItems: 'center',
        justifyContent: 'space-between',
        marginLeft: 12,
        marginRight: 12,
        marginBottom: 0,
    },
    seekbarContainer: {
        alignSelf: 'stretch',
        height: 28,
        marginLeft: 20,
        marginRight: 20,
    },
    seekbarTrack: {
        backgroundColor: SEEKBAR_COLOR,
        height: 3,
        position: 'relative',
        top: 14,
        width: '100%',
    },
    seekbarFill: {
        height: 3,
        width: '100%',
        backgroundColor: darkMode ? 'white' : 'black',
    },
    seekbarHandle: {
        position: 'absolute',
        marginLeft: -7,
        height: 28,
        width: 28,
    },
    seekbarCircle: {
        borderRadius: 12,
        position: 'relative',
        top: 9,
        left: 8,
        height: 12,
        width: 12,
        backgroundColor: darkMode ? 'white' : 'black',
    },
    timeText: {
        color: darkMode ? 'white' : 'black',
    },
}));

export { VideoController };

==> fleetback-media-carousel-1.1.6.tgz: package/src/VideoController.js <==
// @flow
import React, { Component } from 'react';
import { observer } from 'mobx-react';
import { PanResponder, PanResponderInstance, Platform, Text, TouchableHighlight, View } from 'react-native';
import {action, makeObservable, observable} from 'mobx';
import EntypoIcon from 'react-native-vector-icons/Entypo';
import AntDesign from 'react-native-vector-icons/AntDesign';
import MaterialIcons from 'react-native-vector-icons/MaterialIcons';
import { contextualStyleSheet } from 'react-native-contextual-dark-mode';
import { SafeAreaView } from 'react-native-safe-area-context';

import { calculateSeekerPosition, formattedDisplayTime } from './VideoPlayerHelper';

export type VideoInformation = {
    duration: number,
    currentTime: number,
    paused: boolean,
    seekerFillWidth: number,
    seekerPosition: number,
    seekerOffset: number,
    seeking: boolean,
    endReached: boolean,
    target: ?string,
};

type VideoPlayerProps = {
    actionRightCallback?: Function,
    actionRightIcon?: Component,
    actionLeftCallback?: Function,
    actionLeftIcon?: Component,
    isDarkMode?: boolean,
    updateVideoInformation: (id: string, info: VideoInformation) => void,
    videoInformation: VideoInformation,
    onSeek: Function,
    id: string,
    seekerWidth: number,
    setSeekerWidth: Function,
    onEnd: Function,
};

@observer
class VideoController extends Component<VideoPlayerProps> {
    @observable dragging = false;
    seekPanResponder: PanResponderInstance;

    constructor(props) {
        super(props);
        makeObservable(this);
        this.initSeekPanResponder();
    }

    @action
    setIsDragging = () => { this.dragging = true; };

    @action
    setNotDragging = () => { this.dragging = false; };

    initSeekPanResponder = () => {
        this.seekPanResponder = PanResponder.create({
            onStartShouldSetPanResponder: () => true,
            onStartShouldSetPanResponderCapture: () => true,
            onMoveShouldSetPanResponder: () => true,
            onMoveShouldSetPanResponderCapture: () => true,
            onPanResponderTerminationRequest: () => false,
            onPanResponderGrant: (evt) => {
                const { updateVideoInformation, videoInformation, id } = this.props;
                this.setSeekerPosition(evt.nativeEvent.locationX, id);
                updateVideoInformation(id, { seeking: true });
                if (videoInformation.endReached) {
                    updateVideoInformation(id, { endReached: false });
                }
            },
            onPanResponderMove: (evt, gestureState) => {
                const {
                    updateVideoInformation,
                    videoInformation: { seeking, paused, seekerOffset, seekerPosition, duration },
                    id,
                    seekerWidth,
                } = this.props;

                if (!seeking) {
                    updateVideoInformation(id, { seeking: true });
                }
                if (!paused && !this.dragging) {
                    updateVideoInformation(id, { paused: true });
                    this.setIsDragging();
                } else {
                    this.setSeekerPosition(seekerOffset + gestureState.dx, id);
                    const timeToSeek = (duration * (seekerPosition / seekerWidth));
                    this.seekTo(timeToSeek);
                    this.clearControlTimeout();
                }
            },
            onPanResponderRelease: () => {
                const {
                    updateVideoInformation,
                    videoInformation: { paused, seekerPosition, duration },
                    id,
                    seekerWidth,
                } = this.props;

                if (this.dragging && paused) {
                    this.setNotDragging();
                    updateVideoInformation(id, { paused: false });
                }

                const timeToSeek = (duration * (seekerPosition / seekerWidth));

                if (timeToSeek >= duration) {
                    this.props.onEnd();
                } else {
                    this.seekTo(timeToSeek);
                    this.clearControlTimeout();
                    this.seekTimeout = setTimeout(() => {
                        updateVideoInformation(id, { seeking: false });
                    }, 150);
                }
            },
        });
    };

    @action
    togglePlayPause = () => {
        const { updateVideoInformation, videoInformation, id, onSeek } = this.props;
        updateVideoInformation(id, { paused: !videoInformation.paused });
        if (videoInformation.endReached) {
            updateVideoInformation(id, { endReached: false });
            if (Platform.OS === 'ios') {
                onSeek(0);
            }
        }
    };

    @action
    setSeekerPosition = (position: number = 0, id: number, newSeekerWidth?: number) => {
        const { updateVideoInformation, videoInformation, seekerWidth } = this.props;
        const pos = Math.max(0, Math.min((newSeekerWidth ?? seekerWidth), position));
        updateVideoInformation(id, { seekerFillWidth: pos, seekerPosition: pos });
        if (!videoInformation.seeking) {
            updateVideoInformation(id, { seekerOffset: position });
        }
    };

    @action
    seekTo = (time: number = 0) => {
        const { updateVideoInformation, onSeek, id } = this.props;
        updateVideoInformation(id, { currentTime: time });
        onSeek(time);
    };

    onLayout = ({ nativeEvent: { layout: { width } } }) => {
        const { seekerWidth, setSeekerWidth, videoInformation, id } = this.props;
        if (seekerWidth !== width) {
            setSeekerWidth(width);
            if (videoInformation) {
                const position = calculateSeekerPosition(videoInformation.currentTime, videoInformation.duration, width);
                this.setSeekerPosition(position, id, width);
            }
        }
    };

    skipClosure = (skipValue: number) => action(() => {
        const {
            id,
            videoInformation: { currentTime, duration },
            seekerWidth,
            updateVideoInformation,
        } = this.props;
        let endReached = false;

        updateVideoInformation(id, { seeking: true });
        let skippedBack = currentTime + skipValue;
        if (skippedBack < 0) {
            skippedBack = 0;
        } else if (skippedBack > duration) {
            skippedBack = duration;
            endReached = true;
        }
        updateVideoInformation(id, { currentTime: skippedBack, endReached });
        const position = calculateSeekerPosition(skippedBack, duration, seekerWidth);
        this.setSeekerPosition(position, id);
        const timeToSeek = (duration * (position / seekerWidth));
        this.seekTo(timeToSeek);
        this.seekTimeout = setTimeout(() => {
            updateVideoInformation(id, { seeking: false });
        }, 150);
    });

    clearControlTimeout = () => {
        clearTimeout(this.seekTimeout);
    };

    renderSeekbar(color: string, seekerPosition: number) {
        return (
            <View
                style={s.seekbarContainer}
                collapsable={false}
                {...this.seekPanResponder.panHandlers}
            >
                <View
                    style={s.seekbarTrack}
                    onLayout={this.onLayout}
                    pointerEvents="none"
                >
                    <View
                        style={[
                            s.seekbarFill,
                            { width: seekerPosition },
                        ]}
                        pointerEvents="none"
                    />
                </View>
                <View
                    style={[s.seekbarHandle, { left: seekerPosition - 1 }]}
                    pointerEvents="none"
                >
                    <View
                        style={s.seekbarCircle}
                        pointerEvents="none"
                    />
                </View>
            </View>
        );
    }

    renderControl = (callback, children, disabled, testID) => (callback && children ? (
        <TouchableHighlight
            underlayColor="transparent"
            activeOpacity={0.3}
            disabled={disabled}
            onPress={callback}
            style={s.controlButton}
            testID={testID}
        >
            {children}
        </TouchableHighlight>
    ) : (<View style={s.emptyBox} />));

    renderPlayPause = (paused: boolean, color: string) => this.renderControl(
        this.togglePlayPause,
        paused
            ? <EntypoIcon size={33} name="controller-play" color={color} />
            : <AntDesign size={35} name="pause" color={color} />,
        false,
        'VideoController-PlayPause'
    );

    renderBackTime = (color: string, disabled: boolean) => this.renderControl(
        this.skipClosure(-5),
        <MaterialIcons size={35} name="replay-5" color={color} />,
        disabled,
        'VideoController-BackTime'
    );

    renderForwardTime = (color: string, disabled: boolean) => this.renderControl(
        this.skipClosure(5),
        <MaterialIcons size={35} name="forward-5" color={color} />,
        disabled,
        'VideoController-ForwardTime'
    );

    render() {
        const { videoInformation, actionLeftCallback, actionLeftIcon, actionRightCallback, actionRightIcon } = this.props;
        const { currentTime, duration, seekerPosition, paused }: VideoInformation = videoInformation;
        const color = this.props.isDarkMode ? 'white' : 'black';
        const isVideoLongEnough = duration > 6;
        return (
            <SafeAreaView style={s.footerContainer} edges={['bottom', 'right', 'left']} mode="padding">
                <View style={s.column}>
                    {this.renderSeekbar(color, seekerPosition)}
                    <View style={s.timerContainer}>
                        <Text style={s.timeText}>{formattedDisplayTime(currentTime)}</Text>
                        <Text style={s.timeText}>{formattedDisplayTime(duration)}</Text>
                    </View>
                    <View style={s.bottomControlGroup}>
                        {this.renderControl(actionLeftCallback, actionLeftIcon, false, 'VideoController-LeftAction')}
                        {this.renderBackTime(isVideoLongEnough ? color : SEEKBAR_COLOR, !isVideoLongEnough)}
                        {this.renderPlayPause(paused, color)}
                        {this.renderForwardTime(isVideoLongEnough ? color : SEEKBAR_COLOR, !isVideoLongEnough)}
                        {this.renderControl(actionRightCallback, actionRightIcon, false, 'VideoController-RightAction')}
                    </View>
                </View>
            </SafeAreaView>
        );
    }
}

const SEEKBAR_COLOR = '#7D7D81';

const s = contextualStyleSheet((darkMode) => ({
    footerContainer: {
        position: 'absolute',
        bottom: 0,
        width: '100%',
        flexDirection: 'row',
        justifyContent: 'center',
        backgroundColor: darkMode ? '#1D252DB3' : '#EDEDEDB3',
    },
    controlButton: {
        paddingVertical: 16,
    },
    emptyBox: {
        width: 35,
        height: 35,
    },
    column: {
        width: '100%',
        flexDirection: 'column',
        alignItems: 'center',
        justifyContent: 'space-between',
    },
    timerContainer: {
        flexDirection: 'row',
        width: '100%',
        justifyContent: 'space-between',
        paddingHorizontal: 20,
    },
    bottomControlGroup: {
        flexDirection: 'row',
        height: null,
        width: null,
        alignSelf: 'stretch',
        alignItems: 'center',
        justifyContent: 'space-between',
        marginLeft: 12,
        marginRight: 12,
        marginBottom: 0,
    },
    seekbarContainer: {
        alignSelf: 'stretch',
        height: 28,
        marginLeft: 20,
        marginRight: 20,
    },
    seekbarTrack: {
        backgroundColor: SEEKBAR_COLOR,
        height: 3,
        position: 'relative',
        top: 14,
        width: '100%',
    },
    seekbarFill: {
        height: 3,
        width: '100%',
        backgroundColor: darkMode ? 'white' : 'black',
    },
    seekbarHandle: {
        position: 'absolute',
        marginLeft: -7,
        height: 28,
        width: 28,
    },
    seekbarCircle: {
        borderRadius: 12,
        position: 'relative',
        top: 9,
        left: 8,
        height: 12,
        width: 12,
        backgroundColor: darkMode ? 'white' : 'black',
    },
    timeText: {
        color: darkMode ? 'white' : 'black',
    },
}));

export { VideoController };
